#!/usr/bin/env node
"use strict"
const subProcess = require('child_process');
const fs = require('fs'); // file system
const prompt = require('prompt'); // prompt (user input)
// installed moules (global variable)
let DEPENDENCIES = ["bootstrap","popper.js","rytm-webflow","rytm-helpers","rytm-wcag"];
// flexible modules (prompt)
const MODULES_PROMPT = ["gsap","imagesloaded"]
// project directory = current directory
const __PROJ_DIR = process.cwd();

// welcome
const welcome = () => {
  const col = '\x1b[36m%s\x1b[0m'; // cyan
  console.log(col, '									');
  console.log(col, '            ◙◙◙◙◙◙◙◙◙◙◙◙◙    TM 	');
  console.log(col, '           ◙◙◙◙◙◙◙◙◙◙◙◙◙◙◙◙ 		');
  console.log(col, '          ◙◙◙◙◙◙◙◙◙◙◙◙◙◙◙◙◙ 		');
  console.log(col, '         ◙◙◙◙◙ ◙◙◙◙◙◙◙◙◙◙◙ 		');
  console.log(col, '        ◙◙◙◙◙  ◙◙◙◙◙◙◙◙ 			');
  console.log(col, '       ◙◙◙◙◙    ◙◙◙◙◙ 			');
  console.log(col, '      ◙◙◙◙◙      ◙◙◙◙◙ 			');
  console.log(col, '     ◙◙◙◙◙        ◙◙◙◙◙ 			');
  console.log(col, '    ◙◙◙◙◙          ◙◙◙◙◙ 			');
  console.log(col, '									');
}
const goodbye = () => {
  const col = '\x1b[36m%s\x1b[0m'; // cyan
  console.log(col, '									');
  console.log(col, '	All good.	One love! ');
  console.log(col, '									');
}

const isModuleInstalled = (name) => {
  return DEPENDENCIES.indexOf(name) >= 0
}

const getPromptAttrs = () => {
  let props = [
    {
      description: 'Project name',
      name: 'project',
      type: 'string',
      validator: /^[a-zA-Z]+$/,
      warning: 'Project name is not valid, it can only contains letters',
      required: true
    }
  ]
  MODULES_PROMPT.forEach((moduleName) => {
    props.push({
      description: "Include module: " + moduleName,
      name: moduleName,
      type: 'boolean',
      default: false,
    })
  })
  return props
}
// Helper: handle shell exec errors
const handleShellErr = (err, stdout, stderr) => {
  if (err) {
    console.log('###	' + err);
    return false;
  }
  console.log('	' + stdout);
}
/**
 * copy file from template 
 */
const fromTemplate = (from, output, replace) => {
  let content = fs.readFileSync(from, 'utf8')
  replace.forEach((repl) => {
    content = content.split(repl.search).join(repl.replacement)
  })
  fs.writeFileSync(output, content);
}
// random token
const getRandomToken = (length) => {
  let result = '';
  let characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  let charactersLength = characters.length;
  for ( let i = 0; i < length; i++ ) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return result;
}

const buildProject = () => {
  // prompt start
  prompt.start()
  // collect data
  prompt.get(getPromptAttrs(), (err, result) => {
    if (!result) {
      console.log('\x1b[36m%s\x1b[0m', '	Hate to see you leave :( ');
      return false;
    }
    const projectName = result.project
    let promptDependencies = []
    MODULES_PROMPT.forEach((moduleName) => {
      if (result[moduleName] == true) {
        promptDependencies.push(moduleName)
      }
    })
    DEPENDENCIES = [...DEPENDENCIES, ...promptDependencies]
    // # make directories
    fs.mkdirSync(__PROJ_DIR + "/src/scripts", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/src/scss", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/local/config", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/local/models", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/local/images/favicons", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/local/views/_boxes/svg", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/local/views/item", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/tmp/logs", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/tmp/cache", { recursive: true });
    fs.mkdirSync(__PROJ_DIR + "/public/upload/thumb", { recursive: true });
    // chmod
    fs.chmodSync(__dirname + '/public/tmp', 0o777);
    fs.chmodSync(__dirname + '/public/tmp/logs', 0o777);
    fs.chmodSync(__dirname + '/public/tmp/cache', 0o777);
    fs.chmodSync(__dirname + '/public/upload', 0o777);
    fs.chmodSync(__dirname + '/public/upload/thumb', 0o777);
    // Copy files
    // ## .gitignore ##
    fs.copyFileSync(
      __dirname + '/templates/_gitignore', 
      __PROJ_DIR + "/.gitignore"
    )
    // ## .htaccess ##
    fs.copyFileSync(
      __dirname + '/templates/_htaccess', 
      __PROJ_DIR + "/public/.htaccess"
    )
    // ## webpack.config.js ##
    fs.copyFileSync(
      __dirname + '/templates/webpack.config.js', 
      __PROJ_DIR + "/webpack.config.js"
    )
    // ## package.json ##
    fromTemplate(
      __dirname + '/templates/package.json', 
      __PROJ_DIR + '/package.json', 
      [
        {
          search: "<%= project_name %>",
          replacement: projectName
        }
      ]
    )
    // ## index.php ##
    fs.copyFileSync(
      __dirname + '/templates/index.php', 
      __PROJ_DIR + "/public/index.php"
    );
    // ## robots.txt ##
    fs.copyFileSync(
      __dirname + '/templates/robots.txt', 
      __PROJ_DIR + "/public/robots.txt"
    );
    // ## humans.txt ##
    fs.copyFileSync(
      __dirname + '/templates/humans.txt', 
      __PROJ_DIR + "/public/humans.txt"
    );
    // ## SRC files ##
    // all scss files
    fs.cpSync(
      __dirname + '/templates/src/scss', 
      __PROJ_DIR + "/src/scss",
      { recursive: true }
    );
    // app.js
    fs.copyFileSync(
      __dirname + '/templates/src/scripts/app.js', 
      __PROJ_DIR + "/src/scripts/app.js"
    );
    // ## rytm-webflow dependendent 
    if (isModuleInstalled("rytm-webflow")) {
      fs.mkdirSync(__PROJ_DIR + "/src/scripts/controllers", { recursive: true });
      fs.mkdirSync(__PROJ_DIR + "/src/scripts/shared", { recursive: true });
      fs.mkdirSync(__PROJ_DIR + "/src/scripts/views", { recursive: true });
      // copy rytm-webflow related JS files
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/aswap.js', 
        __PROJ_DIR + "/src/scripts/aswap.js"
      );
      // scripts/shared (aswap)
      fs.cpSync(
        __dirname + '/templates/src/scripts/shared', 
        __PROJ_DIR + "/src/scripts/shared",
        { recursive: true }
      );
      // scripts/controllers (aswap) 
      fs.cpSync(
        __dirname + '/templates/src/scripts/controllers', 
        __PROJ_DIR + "/src/scripts/controllers",
        { recursive: true }
      );
      // scripts/views (aswap)
      fs.cpSync(
        __dirname + '/templates/src/scripts/views', 
        __PROJ_DIR + "/src/scripts/views",
        { recursive: true }
      );
    }
    // ### LOCAL ###
    // models
    fs.cpSync(
      __dirname + '/templates/local/models', 
      __PROJ_DIR + "/public/local/models",
      { recursive: true }
    );
    // views
    fs.cpSync(
      __dirname + '/templates/local/views', 
      __PROJ_DIR + "/public/local/views",
      { recursive: true }
    );
    // config files
    fs.copyFileSync(
      __dirname + '/templates/local/config/setup.php', 
      __PROJ_DIR + "/public/local/config/setup.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/config/local-sample.php', 
      __PROJ_DIR + "/public/local/config/local-sample.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/config/imageconfig.php', 
      __PROJ_DIR + "/public/local/config/imageconfig.php"
    )
    // ## config/local.php ##
    fromTemplate(
      __dirname + '/templates/local/config/local.php', 
      __PROJ_DIR + '/public/local/config/local.php', 
      [
        {
          search: "<%= devdbname %>",
          replacement: projectName
        },
        {
          search: "<%= devdbpass %>",
          replacement: projectName
        },
        {
          search: "<%= devtoken %>",
          replacement: getRandomToken(40)
        }
      ]
    )
    // ### npm install
    subProcess.execSync("npm install", handleShellErr);
    subProcess.execSync("npm install --save " + DEPENDENCIES.join(" "), handleShellErr);
    subProcess.execSync("npm run initmodules", handleShellErr);
    subProcess.execSync("npm run build", handleShellErr);
    goodbye();
  }); 
}
// welcome msg
welcome();
// build project
buildProject();